# coding=utf-8
import os, sys
from context import InitContext
from lcyframe.app import App
import platform

"""
定时任务
"""
config = InitContext.get_context()

config["wsgi"]["port"] = 8181 if len(sys.argv) == 1 else int(
    sys.argv[1])  # config["wsgi"]["port"] if len(sys.argv) == 1 else int(sys.argv[1])
config["wsgi"]["debug"] = False
config["wsgi"]["logging_debug"] = "debug"

config["api_docs"]["auto_generate"] = False
config["mqtt_config"]["client_id_name"] = "crontab_%s_%s" % (platform.node(), config["wsgi"]["port"])


config["task"] = [
    (CrontabModel.dosomething, 8000),  # ******
]

if __name__ == "__main__":
    app = App(**config)
    app.start()



# -*- coding: utf-8 -*-

"""HotQueue is a Python library that allows you to use Redis as a message queue
within your Python programs.
"""

from functools import wraps
try:
    import cPickle as pickle
except ImportError:
    import pickle

import redis

def key_for_name(name):
    """Return the key name used to store the given queue name in Redis."""
    return 'hotqueue:%s' % name


class HotQueue(object):

    """Simple FIFO message queue stored in a Redis list. Example:

    :param name: name of the queue
    :param serializer: the class or module to serialize msgs with, must have
        methods or functions named ``dumps`` and ``loads``,
        `pickle <http://docs.python.org/library/pickle.html>`_ is the default,
        use ``None`` to store messages in plain text (suitable for strings,
        integers, etc)
    :param kwargs: additional kwargs to pass to :class:`Redis`, most commonly
        :attr:`host`, :attr:`port`, :attr:`db`
    """

    def __init__(self, name, serializer=pickle, **kwargs):
        self.name = name
        self.serializer = serializer
        self.__redis = redis.Redis(**kwargs)

    def __len__(self):
        return self.__redis.llen(self.key)

    @property
    def key(self):
        """Return the key name used to store this queue in Redis."""
        return key_for_name(self.name)

    def clear(self):
        """Clear the queue of all messages, deleting the Redis key."""
        self.__redis.delete(self.key)

    def consume(self, **kwargs):
        """Return a generator that yields whenever a message is waiting in the
        queue. Will block otherwise. Example:

        >>> for msg in queue.consume(timeout=1):
        ...     print(msg)
        my message
        another message

        :param kwargs: any arguments that :meth:`~hotqueue.HotQueue.get` can
            accept (:attr:`block` will default to ``True`` if not given)
        """
        kwargs.setdefault('block', True)
        try:
            while True:
                msg = self.get(**kwargs)
                if msg is None:
                    break
                yield msg
        except KeyboardInterrupt:
            print; return

    def get(self, block=False, timeout=None):
        """Return a message from the queue. Example:

        >>> queue.get()
        'my message'
        >>> queue.get()
        'another message'

        :param block: whether or not to wait until a msg is available in
            the queue before returning; ``False`` by default
        :param timeout: when using :attr:`block`, if no msg is available
            for :attr:`timeout` in seconds, give up and return ``None``
        """
        if block:
            if timeout is None:
                timeout = 0
            msg = self.__redis.blpop(self.key, timeout=timeout)
            if msg is not None:
                msg = msg[1]
        else:
            msg = self.__redis.lpop(self.key)
        if msg is not None and self.serializer is not None:
            msg = self.serializer.loads(msg)
        return msg

    def put(self, *msgs):
        """Put one or more messages onto the queue. Example:

        >>> queue.put("my message")
        >>> queue.put("another message")

        To put messages onto the queue in bulk, which can be significantly
        faster if you have a large number of messages:

        >>> queue.put("my message", "another message", "third message")
        """
        if self.serializer is not None:
            msgs = map(self.serializer.dumps, msgs)
        self.__redis.rpush(self.key, *msgs)

    def worker(self, *args, **kwargs):
        """Decorator for using a function as a queue worker. Example:

        >>> @queue.worker(timeout=1)
        ... def printer(msg):
        ...     print(msg)
        >>> printer()
        my message
        another message

        You can also use it without passing any keyword arguments:

        >>> @queue.worker
        ... def printer(msg):
        ...     print(msg)
        >>> printer()
        my message
        another message

        :param kwargs: any arguments that :meth:`~hotqueue.HotQueue.get` can
            accept (:attr:`block` will default to ``True`` if not given)
        """
        def decorator(worker):
            @wraps(worker)
            def wrapper(*args):
                for msg in self.consume(**kwargs):
                    worker(*args + (msg,))
            return wrapper
        if args:
            return decorator(*args)
        return decorator


if __name__ == "__main__":
    queue = HotQueue("zaza")
    queue.put({"action": "foo_function", "data": {"foo": "bar"}})


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

class CeleryWorker(object):
    app = None

    def __init__(self, **kwargs):
        """
        消费者守护进程
        :param kwargs:
        """

        celery_config = kwargs.get("celery_config")
        if not celery_config or not celery_config.get("run", True):
            raise Exception("celery_config not found")
        self.name = celery_config.get("work_name_prefix", "celery_works_name")
        self.loglevel = celery_config.get("loglevel", "DEBUG")
        self.queue = celery_config.get("queue", "")    # q1,q2,q3
        self.concurrency = celery_config.get("concurrency")
        self.pool = celery_config.get("pool")

    def start(self):
        """
        Global Options:
          -A APP, --app APP
          -b BROKER, --broker BROKER
          --result-backend RESULT_BACKEND
          --loader LOADER
          --config CONFIG
          --workdir WORKDIR
          --no-color, -C
          --quiet, -q

        :param app:
        :return:
        """
        argv = [
            'worker',
            '--loglevel=%s' % self.loglevel,
            '-n=%s' % self.name,
        ]
        if self.queue:
            argv.append('-Q%s' % self.queue)
        if self.concurrency:
            argv.append('--concurrency=%s' % int(self.concurrency))
        if self.pool:
            argv.append('--pool=%s' % self.pool)
        if self.app is None:
            raise
        self.app.worker_main(argv)



if __name__ == "__main__":
    """
    每个CPU一个主进程，每个主进程开启8个线程并发执行。理论并发数量：CPU * 8。
    设定单个主进程单位时间内的允许执行的任务: --concurrency=10
    celery -A app worker -l info
    """

    CeleryWorker().start()


from lcyframe.nsq_server import NsqWorker
import os
from bson.objectid import ObjectId
from lcyframe import yaml2py
from base import BaseModel
from lcyframe import ServicesFindet
from context import InitContext


config = InitContext.get_context()
config["nsq_config"]["client_id_name"] = "nsq_works_" + str(ObjectId())


w = NsqWorker(**config)
yaml2py.impmodule(BaseModel, "model")
ServicesFindet(w, BaseModel)(config)
w.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Copyright lcyframe 123220663@qq.com
from lcyframe import route
from lcyframe import funts
from lcyframe import BaseHandler, BaseModel as Model


@route("/user")
class UserHandler(BaseHandler):
    """
    用户
    """

    @funts.params()
    def post(self):
        """添加
        测试post
        
        Request extra query params:
        - a # 角色id type: integer
        - b # 供应商id type: string
        - c # 手机号 type: string
        - d # 城市全拼列表 type: int
        - pic # 文件 type: file
        

        :return:
        :rtype:
        """

        self.write_success()

    def get(self):
        """查看
        测试get
        
        Request extra query params:
        - a # 角色id type: integer
        - b # 供应商id type: string
        - d # 城市全拼列表 type: int
        

        :return:
        :rtype:
        """
        pass


@route("/user_list")
class UserlistHandler(BaseHandler):
    """
    用户列表
    """

    def post(self):
        """添加
        测试post
        
        Request extra query params:
        - a # 角色id type: integer
        - b # 供应商id type: string
        - c # 手机号 type: string
        - d # 城市全拼列表 type: int
        - pic # 文件 type: file
        

        :return:
        :rtype:
        """
        pass

    def get(self):
        """查看
        测试get
        
        Request extra query params:
        - a # 角色id type: integer
        - b # 供应商id type: string
        - d # 城市全拼列表 type: int
        

        :return:
        :rtype:
        """
        pass




#!/usr/bin/env python
# -*- coding:utf-8 -*-

import logging
from .base import BaseModel, BaseHandler, BaseSchema
# try:
#     from .mq_server import MqWorker
#     from .libs.mq_route import MqRoute
# except ImportError as e:
#     logging.error(e)
from .libs.mqtt_route import WriteMqtt
from .libs.mqtt_route import MqttTask
from .mqtt_server import MqttWorker, ReadMqtt
from .nsq_server import NsqWorker
from .libs.nsq_route import NsqTask, ReadNsq, WriteNsq
from .celery_server import CeleryWorker
from .libs import celery_route
# from .libs.beanstalk_route import BsRoute
# from .beanstalk_server import BeanstalkWorker
from .libs.route import route
from .libs import funts, utils
from .app import App
from .libs import yaml2py
from .libs.services import ServicesFindet




__all__ = ("App", "yaml2py", "BaseModel", "BaseHandler", "BaseSchema",
           "route", "funts", "utils",
           "ServicesFindet",
           # "MqWorker", "MqRoute",
           "MqttTask", "MqttWorker", "ReadMqtt",  "WriteMqtt",
           "NsqTask", "NsqWorker", "ReadNsq", "WriteNsq",
           "CeleryWorker", "celery_route",
           # "BeanstalkWorker", "BsRoute"
           )


import logging
from logging import handlers

"""
https://www.cnblogs.com/nancyzhu/p/8551506.html
https://www.cnblogs.com/xianyulouie/p/11041777.html
"""

level_mp = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'critical': logging.CRITICAL
    }

class Logger(object):
    instance = {}

    def __init__(self, filename, level='info', when='D', backCount=10, fmt='%(asctime)s - %(pathname)s:%(lineno)d - %(levelname)s: %(message)s'):
        self.logger = logging.getLogger(filename)
        self.format_str = logging.Formatter(fmt)
        self.logger.setLevel(level_mp[level])

        # redirct to console
        console = logging.StreamHandler()
        console.setFormatter(self.format_str)
        self.logger.addHandler(console)

        # wirte to file
        handler = handlers.TimedRotatingFileHandler(filename=filename,      # 保存路径
                                                    when=when,              # 间隔的时间单位，单位有: S秒，M 分，H 小时，D 天，W 每星期（interval==0时代表星期一），midnight 每天凌晨
                                                    backupCount=backCount,  # 文件的个数，如果超过这个个数，就会自动删除
                                                    encoding='utf-8')
        handler.setFormatter(self.format_str)
        self.logger.addHandler(handler)


    @classmethod
    def get_instance(cls, filename, **kwargs):
        file_name = filename.split("/")[-1]
        if file_name not in cls.instance:
            cls.instance[file_name] = Logger(filename, **kwargs)
        return cls.instance[file_name]

if __name__ == '__main__':
    # server
    app = Logger.get_instance('app.log', level='debug')
    app.logger.debug('debug')
    app.logger.info('info')
    app.logger.warning('警告')
    app.logger.error('报错')
    app.logger.critical('严重')

    # db
    db = Logger('error.log', level='error')
    db.logger.error('error')


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import webbrowser, os
from ..base import BaseHandler


class DocsHandler(BaseHandler):
    """
    docs
    """

    def get(self):
        host = self.app_config["wsgi"]["host"]
        if "127.0.0.1" not in host and "localhost" not in host and "0.0.0.0" not in host:
            if "://" in host:
                if "www" not in host:
                    host = host.replace("://", "://docs.")
                else:
                    host = host.replace("www", "docs")
            else:
                if "www" not in host:
                    host = "docs." + host
                else:
                    host = host.replace("www", "docs")
            self.redirect(host)
        else:
            html_path = "index.html" if self.app_config.get("output","html") else "docs/build/html/index.html"
            file_path = "file://"+os.path.realpath("/%s/%s/%s"%(self.app_config["ROOT"],
                                                                self.app_config["api_docs"]["docs_dir"],
                                                                html_path
                                                                ))
            webbrowser.open(file_path)







#!/usr/bin/env python
# -*- coding:utf-8 -*-
import sys, os
root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)
from . import *
from lcyframe.libs import cprint

def login():
    headers = {"app_key": "******"}
    cprint.cprint(admin_login_and_send("post", "/admin/login",
               {"user_name": "admin",
                "pass_word": "123456",
                },
               headers=headers))

def create(user_name, nick_name):
    headers = {"uid": 100000}

    params = {
        "pass_word": "123456",  # "15076349690",
        "user_name": user_name,  # "******",
        "nick_name": nick_name,
        "sex": 1,
        "mobile": "13888888888",
        "email": "123@qq.com",
        "gid": 2
    }

    return admin_login_and_send(
        methed="post",
        url="/admin/member",
        params=params,
        headers=headers,
    )


if __name__ == "__main__":
    login()




from lcyframe.mqtt_server import MqttWorker
import os
from bson.objectid import ObjectId
from lcyframe import yaml2py
from base import BaseModel
from lcyframe import ServicesFindet
from context import InitContext


config = InitContext.get_context()
config["mqtt_config"]["client_id_name"] = "mqtt_works_" + str(ObjectId())
config["mqtt_config"]["client_role"] = "worker"

w = MqttWorker(**config)
yaml2py.impmodule(BaseModel, "model")
ServicesFindet(w, BaseModel)(config)
w.start()



# -*- coding: utf-8 -*-
import os
import logging
from celery import Task
from celery import Celery, shared_task
from celery.utils.log import get_task_logger

logger = get_task_logger(__name__)


class MyCelery(Celery):
    def gen_task_name(self, name, module):
        """
        重新命名
        python3.6.5下, 在pycharm里debug模式运行，存在BUG:Fatal Python error: PyCOND_WAIT(gil_cond) failed
        :param name:
        :param module:
        :return:
        """
        module = module.split(".")[-1]
        # if module.endswith('.tasks'):
        #     module = module[:-6]
        return super(MyCelery, self).gen_task_name(name, module)


class Events(object):
    """
    生产者
    """

    # tasks_dict = {}
    #
    # def __setattr__(self, key, value):
    #     self.tasks_dict[key] = value

    def __getattr__(self, name):
        if name not in self.__dict__:
            raise Exception("task '%s' is not exists" % name)
        else:
            return self.__dict__[name]


# 统一处理成功、失败、重试的基类
class BaseTask(Task):
    abstract = True

    def on_success(self, retval, task_id, args, kwargs):
        print('task done: {0}'.format(retval))
        return super(BaseTask, self).on_success(retval, task_id, args, kwargs)

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        # 捕获异常信息 sentrycli.captureException(exc)
        super(BaseTask, self).on_failure(exc, task_id, args, kwargs, einfo)

    def on_retry(self, exc, task_id, args, kwargs, einfo):
        # sentrycli.captureException(exc)
        super(BaseTask, self).on_retry(exc, task_id, args, kwargs, einfo)

    def after_return(self, status, retval, task_id, args, kwargs, einfo):
        # print('Task returned: {0!r}'.format(self.request))
        logger.info("Task returned")

    @classmethod
    def backoff(cls, attempts):
        """
        重试间隔根据次数递增
        1, 2, 4, 8, 16, 32, ...
        """
        return 2 ** attempts


class BaseEvent(object):
    queue = ""




def shared_task(*args, **kwargs):
    """
    接管装饰
    :param args:
    :param kwargs:
    :return:
    """
    return shared_task(*args, **kwargs)



# /usr/env/bin python
# -*- coding:utf-8 -*-
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))


# xxx_handler xx leng must > 1.
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue
    logging.warn("handler [%s] register success!" % hander_file)
    __import__(model_name, globals(), locals(), [model_name], -1)



# -*- coding:utf8 -*-

# 七牛上传类型
FILE_TYPE_AVATAR = 1
FILE_TYPE_BACKIMG = 2
FILE_TYPE_POST = 3  # 以后所有新增的类型,都用这个,不在添加新类型
FILE_TYPE_AUDIO = 4
FILE_TYPE_VIDEO = 5
FILE_TYPE_COVER = 6
FILE_TYPE_PACK = 7
FILE_TYPE_AD = 8
FILE_TYPE_REPLAY = 9
FILE_TYPE_REPLAY_IMG = 10
FILE_TYPE_REPLAY_PPT = 11
FILE_TYPE_REPLAY_LYRIC = 12
FILE_TYPE_STORE = 13  # 商城图片

# 本地TYPE上传类型
LOCAL_TYPE_AUDIO = 0
LOCAL_TYPE_VIDEO = 1
LOCAL_TYPE_COVER = 2
LOCAL_TYPE_AVATAR = 4
LOCAL_TYPE_BACKIMG = 5
LOCAL_TYPE_REPLAY = 6

# 图片尺寸约定
MICRO = 0
SMALL = 1
BIG = 2
SLIDE = 3
WORKS = 4
CHANLLENGE = 5
LARGE = 6
HOME = 7
MAX = 8
RAW = 9
NEW_CHALLENGE = 10
NEW_CHALLENGE2 = 11
AD_START = 12
ALBUM_COVER = 13  # 推荐专辑列表封面图,400*200

# 上传类型对应bucket:(空间名, 域名路径目录)
BUCKET_MAP = {FILE_TYPE_AVATAR: ("iguitar-image", "avatar"),
              FILE_TYPE_BACKIMG: ("iguitar-image", "back"),
              FILE_TYPE_COVER: ("iguitar-image", "cover"),
              FILE_TYPE_POST: ("iguitar-image", "image"),
              FILE_TYPE_STORE: ("iguitar-image", "store"),  # 商城图片
              FILE_TYPE_AUDIO: ("iguitar-audio", "audio"),
              FILE_TYPE_VIDEO: ("iguitar-media", "video"),
              FILE_TYPE_PACK: ("iguitar-bgt-apk", "pack"),
              FILE_TYPE_AD: ("iguitar-image", "ad"),
              FILE_TYPE_REPLAY: ("iguitar-replay", "replay"),
              FILE_TYPE_REPLAY_IMG: ("iguitar-replay", "image"),
              }


# 缩略图尺寸,先缩略再上传
THUMB_SIZE = {MICRO: (100, 100),
              SMALL: (120, 120),
              BIG: (240, 240),
              LARGE: (750, 750),
              SLIDE: (750, 280),
              WORKS: (750, 450),
              CHANLLENGE: (750, 320),
              MAX: (1024, 1024),
              HOME: (750, 580),
              }

QINIU_IMAGE = "http://img.iguitar.immusician.com/"    # "http://7xlomp.com1.z0.glb.clouddn.com/"
QINIU_MEDIA = "http://media.iguitar.immusician.com/"      # "http://7xlizu.com5.z0.glb.clouddn.com/"
QINIU_AUDIO = "http://audio.iguitar.immusician.com/"  # "http://7xloms.com5.z0.glb.clouddn.com/"
QINIU_BGT = "http://bgt.iguitar.immusician.com/"       # "http://7xlomr.com5.z0.glb.clouddn.com/"
QINIU_REPLAY = "http://replay.iguitar.immusician.com/"  # "http://7xrxp6.com1.z0.glb.clouddn.com/"


THUMB_MAP = {MICRO: "micro",
             SMALL: "small",
             BIG: "big",
             LARGE: "large",
             SLIDE: "slide",
             WORKS: "works",
             CHANLLENGE: "chanllenge",
             NEW_CHALLENGE: "newchallenge",
             NEW_CHALLENGE2: 'newchallenge2',
             HOME: "home",
             RAW: "raw",
             AD_START: 'adstart',      # 750*1334
             ALBUM_COVER: 'albumcover'  # # 推荐专辑列表封面图,40*200
             }




from lcyframe import MqttTask
from lcyframe import ReadMqtt
from lcyframe import BaseModel

@MqttTask(topic="#", qos=2)
class ExampleEvent2(ReadMqtt, BaseModel):

    @classmethod
    def on_test_mqtt(cls, **msg):
        print(msg, "this is mqtt message")



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.igGenerator_schema import IdGeneratorSchema

class IdGeneratorModel(BaseModel, IdGeneratorSchema):
    @classmethod
    def gen_uid_id(cls, seq=1):
        """自动分配UID"""
        z = cls.find_and_modify({}, {'$inc': {"uid": seq}})["uid"]
        return z

    @classmethod
    def gen_order_id(cls, prefix="O", seq=1):
        """分配订单ID"""
        pass

    @classmethod
    def gen_company_id(cls, prefix="CD", seq=1):
        """分配企业ID"""
        z = cls.find_and_modify({}, {'$inc': {"company_id": seq}})["company_id"]
        return "%s%07d" % (prefix, z)

    @classmethod
    def gen_agent_id(cls, prefix="A", seq=1):
        """分配代理ID"""
        z = cls.find_and_modify({}, {'$inc': {"agent_id": seq}})["agent_id"]
        return "%s%07d" % (prefix, z)




#!/usr/bin/env python
# -*- coding:utf-8 -*-

from kombu import Exchange, Queue

# https://www.cnblogs.com/yunlongaimeng/p/11121294.html

# 该框架建议使用celery=4.4.7版本

BROKER_URL = "redis://localhost:6379/0"
CELERY_RESULT_BACKEND = "redis://localhost:6379/0"

# 任务失败或超时自动确认，默认为True
CELERY_ACKS_ON_FAILURE_OR_TIMEOUT = False

# 任务完成之后再确认
CELERY_ACKS_LATE = True

# worker进程崩掉之后拒绝确认
CELERY_REJECT_ON_WORKER_LOST = True

# 设置默认不存结果(保存在redis内),如果需要保持结果，请在任务中单独指定。@app.task(ignore_result=False)
CELERY_IGNORE_RESULT = True

# 设置结果的保存时间
CELERY_TASK_RESULT_EXPIRES = 24 * 3600

CELERYD_MAX_TASKS_PER_CHILD = 100  # 每个worker执行了多少任务就会死掉，防止OOM

CELERY_QUEUES = (
    Queue("celery", Exchange("celery", type='direct'), routing_key="celery"),
    Queue("for_task_A", Exchange("for_task_A", type='direct'), routing_key="for_task_A"),
    Queue("for_task_B", Exchange("for_task_B", type='direct'), routing_key="for_task_B")
)

CELERY_ROUTES = {
    'tasks.task_x': {"queue": "celery"},
    'tasks.taskA': {"queue": "for_task_A", "routing_key": "for_task_A"},
    'tasks.taskB': {"queue": "for_task_B", "routing_key": "for_task_B"}
}

# 新增加的定时任务部分：celery默认时区为世界标准时间，比东八区慢8小时
# https://www.cnblogs.com/belingud/p/11716303.html
CELERY_TIMEZONE = 'Asia/Shanghai'

# CELERYBEAT_SCHEDULE = {
#     'taskA_schedule' : {
#         'task':'tasks.taskA',
#         'schedule':2,
#         'args':(5,6)
#     },
#     'taskB_scheduler' : {
#         'task':"tasks.taskB",
#         "schedule":10,
#         "args":(10,20,30)
#     },
#     'add_schedule': {
#         "task":"tasks.add",
#         "schedule":5,
#         "args":(1,2)
#     }
# }



# coding=utf-8
import json
import logging
import time
import urllib
from tornado.gen import Return, coroutine
from tornado.httpclient import HTTPRequest, AsyncHTTPClient
from hashlib import md5


class MobSMS(object):
    """短信验证码"""

    def __init__(self, **kwargs):
        self.debug = kwargs.get("debug", False)
        self.SMS_APP_KEY = kwargs["app_key"]
        self.SMS_APP_SECRET = kwargs["app_secret"]
        self.SEND_URL = kwargs["send_url"]
        self.SMS_VERIFY_URL_CLIENT = kwargs["client_verify_url"]
        self.SMS_VERIFY_URL_SERVER = kwargs["server_verfiy_url"]
        self.SALT = kwargs["salt"]  # 防刷码

    def check_seq(self, **kwargs):
        if self.debug:
            return True

        if "seq" not in kwargs or "timestamp" not in kwargs:
            # 缺少seq
            return False

        now = time.time()
        seq = kwargs["seq"]
        timestamp = kwargs["timestamp"]

        if now - 300 < timestamp < now + 300:
            real_seq = self.gen_seq(**kwargs)
            if real_seq == seq:
                return True

        return False

    def gen_seq(self, **kwargs):
        kwargs.pop("seq", None)

        d = []
        for k in sorted(kwargs.keys()):
            v = kwargs[k]
            if isinstance(v, unicode):
                v = v.encode("u8")
            d.append("%s=%s" % (k, str(v)))
        s = self.SALT + "&".join(d)
        return md5(s).hexdigest()

    @coroutine
    def send_sms(self, zone, phone):
        body = {"appkey": self.SMS_APP_KEY, "zone": zone, "phone": phone}
        r = yield self.mob_send(self.SEND_URL, body)
        raise Return(r)

    @coroutine
    def verify_sms(self, zone, phone, code, is_client=True):
        if self.debug:
            raise Return(True)

        url = self.SMS_VERIFY_URL_CLIENT if is_client else self.SMS_VERIFY_URL_SERVER
        body = {"appkey": self.SMS_APP_KEY, "zone": zone, "phone": phone, 'code': code}
        r = yield self.mob_send(url, body)
        raise Return(r)

    @coroutine
    def mob_send(self, url, body):
        if self.debug:
            raise Return(True)
        request = HTTPRequest(url, method="POST", body=urllib.urlencode(body), validate_cert=False)
        httpClient = AsyncHTTPClient()
        response = yield httpClient.fetch(request)
        if json.loads(response.body).get("status") == 200:
            raise Return(True)
        else:
            print(response.body)
            raise Return(False)


if __name__ == "__main__":
    from tornado.ioloop import IOLoop

    # assert MobSMS.check_seq(seq=u"7170d06b94b6d585ee73a43bf714ecf6", avatar=u"adssfsdfdf", nickname=u"sdfsfsdfdsf",
    #                      sex=123, age="13", login_type="中文123", access_token=u"sdfsdfsdf",
    #                      openid=u"中文123", timestamp=123123213) == 0
    loop = IOLoop.instance()


    def down_callback(future):
        loop.add_callback(loop.stop)


    MobSMS.send_sms(86, 13681236816).add_done_callback(down_callback)
    loop.start()




# coding=utf-8
from base import BaseModel
from lcyframe import yaml2py
from lcyframe import ServicesFindet
from lcyframe.beanstalk_server import BeanstalkWorker
from context import InitContext

config = InitContext.get_context()

tasks = BeanstalkWorker(**config)
yaml2py.impmodule(BaseModel, model_dir="model")
ServicesFindet(tasks, BaseModel)(config)
tasks.start()



# coding=utf-8
from functools import wraps

import re
import types
from lcyframe.libs import utils
import logging

from lcyframe.libs import errors
from lcyframe.libs.JWT import JwtToken
import urllib

def authorize(method):
    @wraps(method)
    def check_token(self, *args, **kwargs):
        _check_token(self)
        return method(self, *args, **kwargs)

    return check_token


def _check_token(self):
    jwt = JwtToken(self.uid, self.application.token_config["secret"], self.application.token_config["expire"])
    try:
        if not jwt.is_validate(self.token):
            raise errors.ErrorTokenInvalid

        if jwt.is_expire(self.token):
            raise errors.ErrorTokenExpireInvalid

        payload = jwt.decode(self.token)
        if payload and payload["uid"] != self.uid:
            raise errors.ErrorTokenInvalid
    except Exception as e:
        raise errors.ErrorTokenInvalid


def admin(method):
    @wraps(method)
    def has_role(self, *args, **kwargs):
        uid = int(self.request.headers.get("uid", None) or 0)
        if not self.application.mongo.admin.find_one({"uid": int(uid)}):
            raise errors.ErrorInvalid

        return method(self, *args, **kwargs)

    return has_role


def params(permission_bit=0):
    def wrap(method):
        @wraps(method)
        def has_role(self, *args, **kwargs):
            self.params = get_params(self)
            if self.app_config.get("logging_config", {}).get("level", "debug") == "debug":
                logging.debug(self.params)

            uid = self.request.headers.get("uid", None) or 0
            if type(uid) != int and uid.isdigit():
                self.uid = int(uid)

            self.token = self.request.headers.get("token", "")

            token_debug = self.token_config.get("debug", False)
            if not token_debug:
                if permission_bit != 0:
                    if not self.uid or not self.token:
                        raise errors.ErrorTokenInvalid

                    _check_token(self)

            return method(self, *args, **kwargs)

        return has_role

    return wrap


def get_params(self, fill_default=True):

    p = {}
    method = self.request.method.lower()

    if not self.request.arguments:
        try:
            body_arguments = {k: v for k, v in
                              [item.split("=") for item in self.request.body.decode().split("&")]}
        except Exception as e:
            body_arguments = {}
    else:
        body_arguments = self.request.arguments

    if not self.request.files:
        files = {}
    else:
        files = self.request.files

    params_mp = {}
    if self.request.path in self.application.api_schema_mp:
        params_mp = self.application.api_schema_mp[self.request.path].get("method", {})
    else:
        # from tornado.routing import RuleRouter, PathMatches, _unquote_or_none
        for route in self.application.handlers:
            match = route.regex.match(self.request.path)        # ref: routing.PathMatches.match
            if match is not None:
                pattern1 = route.regex.pattern
                pattern2 = route.regex.pattern.rstrip("$")
                if pattern1 in self.application.api_schema_mp:
                    params_mp = self.application.api_schema_mp[pattern1].get("method", {})
                elif pattern2 in self.application.api_schema_mp:
                    params_mp = self.application.api_schema_mp[pattern2].get("method", {})
                else:
                    params_mp = {}

                # path_args = [_unquote_or_none(s) for s in match.groups()]
                break

    if method in params_mp:
        for item in params_mp[method].get("parameters", []) or []:
            params_name = item["name"]
            _type = item["type"]

            if _type is [True, False, None]:
                # raise errors.ErrorArgumentType
                msg = "Argument '%s' Type can not is bool." % params_name
                e = errors.ErrorArgumentType
                e.message = msg
                raise e

            required = item.get("required", False)

            regex = item.get("regex")
            allow = item.get("allow", []) or item.get("allowed", [])
            message = item.get("message", "")

            if item.get("in") is None:
                if self.request.arguments:
                    if item["type"] == "list":
                        # 参数为数组
                        params_vaule = self.get_arguments(params_name) or None
                    else:
                        params_vaule = self.get_argument(params_name, None, strip=False)
                else:
                    if not body_arguments:
                        raise errors.ErrorArgumentValue

                    params_vaule = body_arguments.get(params_name)
                    try:
                        if params_vaule:
                            params_vaule = urllib.parse.unquote(params_vaule)
                    except Exception as e:
                        msg = "Argument '%s' Value '%s' Error" % (params_name, params_vaule)
                        e = errors.ErrorArgumentValue
                        e.message = msg
                        raise e
            else:
                _in = item.get("in", "query")
                if _type in ["file", "files"]:
                    _in = item.get("in", "form-data")

                if _in in ["query"]:
                    if item["type"] == "list":
                        # 参数为数组
                        params_vaule = self.get_arguments(params_name) or None
                    else:
                        params_vaule = self.get_argument(params_name, None, strip=False)

                elif _in in ["www-form", "body-form"]:
                    if item["type"] == "list":
                        # 参数为数组
                        params_vaule = self.get_arguments(params_name) or None
                    else:
                        params_vaule = self.get_argument(params_name, None, strip=False)

                elif _in in ["body-json"]:
                    if self.request.arguments:
                        if item["type"] == "list":
                            # 参数为数组
                            params_vaule = self.get_arguments(params_name) or None
                        else:
                            params_vaule = self.get_argument(params_name, None, strip=False)
                    else:
                        if self.request.body:
                            body_arguments = {k: v for k, v in [item.split("=") for item in self.request.body.decode().split("&")]}
                        else:
                            body_arguments = {}

                        params_vaule = body_arguments.get(params_name)
                        try:
                            if params_vaule:
                                params_vaule = urllib.parse.unquote(params_vaule)
                        except Exception as e:
                            msg = "Argument '%s' Value '%s' Error" % (params_name, params_vaule)
                            e = errors.ErrorArgumentValue
                            e.message = msg
                            raise e

                elif _in in ["form-data"]:
                    if item["type"] == "list":
                        # 参数为数组
                        params_vaule = self.get_arguments(params_name) or None
                    elif item["type"] in ["files", "file"]:
                        _type = "dict"
                        if params_name not in files:
                            params_vaule = None
                        else:
                            params_vaule = files[params_name][0]
                    else:
                        params_vaule = self.get_argument(params_name, None, strip=False)

                elif _in in ["binary"]:
                    msg = "参数：'%s'，不再支持将参数位置设置为binary方式" % params_name
                    e = errors.ErrorArgumentValue
                    e.message = msg
                    raise e
                    _type = "str"
                    params_vaule = self.request.body

                else:
                    # raise errors.ErrorArgumentType
                    msg = "Missing Argument '%s'. must in [query, form-data, www-form, file]." % params_name
                    e = errors.ErrorMissingArgument
                    e.message = msg
                    raise e

            if params_vaule is None:
                if required is True:  # 必传
                    msg = "Missing Argument '%s'" % params_name
                    e = errors.ErrorMissingArgument
                    e.message = msg
                    raise e

                if not fill_default:
                    continue

                # 客户端没有传值, 按照默认值类型赋值
                # params_vaule = default() if isinstance(default, types.FunctionType) or isinstance(default, types.TypeType) else default
                if item.get("default") is not None:
                    params_vaule = item.get("default")
                else:
                    continue

            else:
                # 按照默认值的类型 转换参数
                try:
                    params_vaule = utils.TypeConvert.convert_params(_type, params_vaule)
                except Exception as e:
                    # raise errors.ErrorArgumentType
                    msg = "Argument '%s' Type Error, Require '%s'" % (params_name, _type)
                    e = errors.ErrorArgumentType
                    e.message = msg
                    raise e

                if allow and params_vaule not in allow:
                    msg = "Argument '%s' must be in %s" % (params_name, utils.to_json(allow))
                    e = errors.ErrorArgumentValue
                    e.message = message or msg
                    raise e

                if regex and params_vaule is not None and not re.match(r"%s" % regex, str(params_vaule)):
                    # raise errors.ErrorArgumentValue
                    msg = "Argument '%s' Value does not match expression" % params_name
                    e = errors.ErrorArgumentValue
                    e.message = message or msg
                    raise e

            p[params_name] = params_vaule

    return p


def parse_vals(response_key, response, definition):
    t = type(definition)
    if t not in [str]:
        try:
            return t(response)
        except Exception as exc:
            e = errors.ErrorResponse()
            e.message = "KEY '%s' type is Error in Reference .yml response definition! error.message=%s" % (
                response_key, exc.message)
            logging.warning("=== response_key: %s, response: %s ===" % (response_key, response))
            raise e

    for _type in utils.TypeConvert.STR2TYPE.keys():
        value_type = "|%s" % _type
        if value_type in definition and definition.split("|")[-1] == _type:
            try:
                response = utils.TypeConvert.MAP[utils.TypeConvert.STR2TYPE[_type]](response)
            except ValueError as e:
                e = errors.ErrorResponse()
                e.message = "Key '%s' type in response must is %s. Reference .yml response definition!" % (
                    response_key, type(definition))
                raise e
            break

    return response


def recursion(response, definition, force_return=False):
    p = {}
    l = []
    is_dict = True

    e = errors.ErrorResponse()
    if type(response) != type(definition):
        try:
            response = response.encode("u8")
            definition = definition.encode("u8")
        except:
            e.message = "Response Key '%s' data type != .yml response definition!" % utils.to_json(response)
            raise e
            # raise TypeError("Response data type != .yml response definition!")

    if type(definition) is dict:
        for k, v in definition.items():
            k = str(k)
            if k not in response:
                if force_return:
                    e.message = "Key '%s' not in response data. Reference .yml response definition!" % k
                    raise e
                    # raise KeyError("Key %s not in your response data. Reference .yml response definition!" % k)
                else:
                    continue

            # if type(response[k]) in [dict, list, tuple] and type(v) not in [dict, list, tuple]:
            #     e.message = "response Key '%s' must be the same in Reference .yml response definition!" % (k, type(v), type(response[k]))
            #     raise e
            #
            # if type(v) in [dict, list, tuple] and type(response[k]) not in [dict, list, tuple]:
            #     e.message = "response Key '%s' must be the same in Reference .yml response definition!" % (k, type(v), type(response[k]))
            #     raise e

            if type(v) in [dict, list, tuple]:
                if type(response[k]) != type(v):
                    e.message = "Key '%s' in response must is %s. Reference .yml response definition!" % (
                        k, type(v))
                    raise e
                # p[k] = response[k]
                p[k] = recursion(response[k], v, force_return)
            else:
                p[k] = parse_vals(k, response[k], v)
    else:
        is_dict = False
        for index, d in enumerate(response):
            if type(d) not in [dict, list, tuple]:
                l.append(d)
            else:
                is_same_type = True
                first_type = type(definition[0])
                for item in definition:
                    if type(item) != first_type:
                        is_same_type = False
                        break

                # 列表的项类型相同，推荐
                if is_same_type:
                    l.append(recursion(d, definition[0], force_return))
                # 各项类型不相同，需要定义的长度和返回的一致
                elif len(response) == len(definition):
                    l.append(recursion(d, definition[index], force_return))
                else:
                    raise
    return p if is_dict else l


# def recursion(response, definition, force_return=False):
#     """
#     definition: 1
#     definition: s
#     definition: [1, s, [], {}]
#     definition: {k: 1, k: s, k: [], k: {}}
#     :param response:
#     :param definition:
#     :param force_return:
#     :return:
#     """
#     return_data = {}
#     return_dict = {}
#     return_list = []
#     return_static = ""
#     r_type = type(response)
#     d_type = type(definition)
#     e = errors.ErrorResponse()
#
#     check_type(response, definition)
#
#     if d_type is dict:
#         for k, v in definition.items():
#             k = str(k)
#             if k not in response:
#                 if force_return:
#                     e.message = "Key '%s' not in response data. Reference .yml response definition!" % k
#                     raise e
#                 else:
#                     continue
#
#             try:
#                 response[k] = response[k].encode("u8")
#                 v = v.encode("u8")
#             except:
#                 pass
#
#             if type(response[k]) != type(v) and "|" not in v:
#                 e.message = "Key '%s' in response must is %s. Reference .yml response definition!" % (
#                     k, type(v))
#                 raise e
#
#             return_data[k] = recursion(response[k], v, force_return)
#
#     elif d_type in [list, tuple]:
#         # 列表里设置的每个项类型相同
#         is_same_type = True
#         first_type = type(definition[0])
#         for item in definition:
#             if type(item) != first_type:
#                 is_same_type = False
#                 break
#
#         # 各项类型不相同，需要定义的长度和返回的一致
#         if not is_same_type and len(response) != len(definition):
#             e.message = "definition list is_same_type and length != response. Reference .yml"
#             raise e
#
#         for i, v in enumerate(response):
#
#             if is_same_type:
#                 return_list.append(recursion(v, definition[0], force_return))
#             else:
#                 return_list.append(recursion(v, definition[i], force_return))
#     else:
#         return_static = parse_vals(response, response, definition)
#
#     return return_data if return_data else return_list if return_list else return_static
#
#
# def check_type(response, definition):
#     r_type = type(response)
#     try:
#         definition, t_type = definition.split("|")
#     except:
#         t_type = None
#
#     if t_type and t_type in utils.TypeConvert.STR2TYPE:
#         d_type = utils.TypeConvert.STR2TYPE[t_type]
#     else:
#         d_type = type(definition)
#
#     if r_type != d_type:
#         try:
#             response = response.encode("u8")
#             definition = definition.encode("u8")
#         except:
#             e.message = "Response Key '%s' data type != .yml response definition!" % utils.to_json(response)
#             raise e
#
def get_return(self, response):
    method = self.request.method.lower()
    path = self.request.path
    api_schema_mp = self.application.api_schema_mp
    if path not in api_schema_mp or method not in api_schema_mp[path]["method"]:
        return response

    if not api_schema_mp[path]["method"].get(method):
        return response

    definition = api_schema_mp[path]["method"][method].get("responses")
    if not definition:
        return response

    p = recursion(response, definition, self.application.api_docs.get("force_return", False))
    return p


#!/usr/bin/env python
# -*- coding:utf-8 -*-

from lcyframe import BaseHandler as Handler
from lcyframe import BaseModel as Model
from lcyframe import BaseSchema as Schema
from utils import errors, helper, keys

class BaseHandler(Handler):
    """
    This is the base class RequestHandler you apply
    You can rewrite the function you want and inherit the frame parent class
    """
    api_error = errors
    helper = helper
    keys = keys

    def write(self, chunk):
        if type(chunk) is dict and chunk.get("code") == 0:
            chunk['code'] = 200
        return super(BaseHandler, self).write(chunk)

class BaseModel(Model):
    """
    This is the base class Model you apply
    You can rewrite the function you want and inherit the frame parent class
    """
    api_error = errors
    helper = helper
    keys = keys


class BaseSchema(Schema):

    @classmethod
    def shard_rule(cls, shard_key_value):
        """
        :param shard_key_value:
        :return: table_value

        This is default rule by `mod10`
        you can rewrite like this

        :example :: DemoSchema.yml
            def shard_rule(shard_key_value):
                do_your_thing
                ...
        """
        return cls.mod10(shard_key_value)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base64 import b64encode
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA
from base64 import b64decode
from Crypto.PublicKey import RSA
# key = RSA.generate(1024)
# f = open('myPrivateKey.pem','w')
# myPrivateKey = key.exportKey('PEM', pkcs=8)   # 支持pkcs1 pkcs8
# f.write(myPrivateKey)
# f.close()

def encrypt(message):
    externKey="./myPublicKey.pem"
    privatekey = open(externKey, "r", encoding='utf-8')
    encryptor = RSA.importKey(privatekey,)
    encriptedData=encryptor.encrypt(message, 0)
    file = open("./cryptThingy.txt", "wb", encoding='utf-8')
    file.write(encriptedData[0])
    file.close()

def decrypt():
    externKey="./myPrivateKey.pem"
    publickey = open(externKey, "r", encoding='utf-8')
    decryptor = RSA.importKey(publickey)
    retval=None
    file = open("./cryptThingy.txt", "rb", encoding='utf-8')
    retval = decryptor.decrypt(file.read())
    file.close()
    return retval


def rsa_sign(message):
    private_key_file = open('./myPrivateKey.pem', 'r', encoding='utf-8')
    private_key = RSA.importKey(private_key_file)
    hash_obj = SHA.new(message)
    signer = PKCS1_v1_5.new(private_key)
    d = b64encode(signer.sign(hash_obj))
    file = open('./signThing.txt', 'wb', encoding='utf-8')
    file.write(d)
    file.close()

def rsa_verify(message):
    public_key_file = open('./myPublicKey.pem', 'r', encoding='utf-8')
    public_key = RSA.importKey(public_key_file)
    sign_file = open('./signThing.txt', 'r', encoding='utf-8')
    sign = b64decode(sign_file.read())
    h = SHA.new(message)
    verifier = PKCS1_v1_5.new(public_key)
    return verifier.verify(h, sign)

if '__main__' == __name__:
    pass
    # encryptedThingy = encrypt("Loren ipsum")
    # decryptedThingy = decrypt()
    # print("Decrypted: %s" % decryptedThingy)
    #
    rsa_sign('zhangshibo')
    print(rsa_verify('zhangshibo'))


#!/usr/bin/env python
# -*- coding:utf-8 -*-

from lcyframe import route
from lcyframe import funts
from base import BaseHandler

@route("/admin/member")
class AdminHandler(BaseHandler):
    """
    后台管理员
    """

    
    @funts.admin
    @funts.params()
    def put(self):
        """无说明
        None
        
        Request extra query params:
        - uid # None type: int
        - nick_name # None type: str
        - sex # 1 无说明 0 无说明 type: int
        - mobile # 无说明 type: str
        - email # 无说明 type: str
        - gid # 无说明 1 无说明 2 无说明 type: int
        

        :return:
        :rtype:
        """
        
        pass
        

    
    @funts.admin
    @funts.params()
    def post(self):
        """无说明
        None
        
        Request extra query params:
        - user_name # None type: str
        - pass_word # 无说明，无说明 type: str
        - nick_name # None type: str
        - sex # 1 无说明 0 无说明 type: int
        - mobile # 无说明 type: str
        - email # 无说明 type: str
        - gid # 无说明 1 无说明 2 无说明 type: int
        

        :return:
        :rtype:
        """
        
        pass
        

    
    @funts.admin
    @funts.params()
    def get(self):
        """无说明
        None
        
        Request extra query params:
        - uid # uid type: int
        

        :return:
        :rtype:
        """
        
            
        data = self.model.AdminModel.get_admin(self.params.get("_id", ""))
        self.write_success(data=data)
            
        

    
    @funts.admin
    @funts.params()
    def delete(self):
        """无说明
        None
        
        Request extra query params:
        - uid # uid type: int
        

        :return:
        :rtype:
        """
        
        self.model.AdminModel.delete(self.params["_id"])
        self.write_success()
        


@route("/admin/states")
class AdminStatesHandler(BaseHandler):
    """
    无说明/无说明
    """

    
    @funts.admin
    @funts.params()
    def post(self):
        """无说明/无说明
        None
        
        Request extra query params:
        - uid # uid type: int
        - state # 1 无说明 -1 无说明 type: int
        

        :return:
        :rtype:
        """
        
        pass
        


@route("/admin/login")
class AdminLoginHandler(BaseHandler):
    """
    无说明
    """

    
    @funts.admin
    @funts.params()
    def post(self):
        """无说明
        None
        
        Request extra query params:
        - user_name # 无说明 type: str
        - pass_word # 无说明 type: str
        

        :return:
        :rtype:
        """
        
        pass
        


@route("/admin/find")
class AdminFindHandler(BaseHandler):
    """
    无说明
    """

    
    @funts.admin
    @funts.params()
    def post(self):
        """无说明
        无说明（无说明）
        
        Request extra query params:
        - page # 无说明 type: str
        

        :return:
        :rtype:
        """
        
        pass
        

    
    @funts.admin
    @funts.params()
    def get(self):
        """无说明
        无说明
        

        :return:
        :rtype:
        """
        
            
        data = self.model.AdminModel.get_admin(self.params.get("_id", ""))
        self.write_success(data=data)
            
        


@route("/admin/resetpwd")
class ResetpwdHandler(BaseHandler):
    """
    无说明
    """

    
    @funts.admin
    @funts.params()
    def post(self):
        """无说明
        None
        
        Request extra query params:
        - pass_word # 无说明 type: str
        - new_pass_word # 无说明 type: str
        

        :return:
        :rtype:
        """
        
        pass
        




from lcyframe import NsqTask
from lcyframe import ReadNsq
from lcyframe import BaseModel

@NsqTask(topic="test", channel="channel2")
class ExampleEvent2(ReadNsq, BaseModel):

    @classmethod
    def on_create_user(cls, **msg):
        print(msg, "channel2")
        return True     # 必须返回True False None之一

